@@ -226,6 +226,11 @@ static const char *parse_sysv_option(void){
     // This list of options is ONLY for those defined by the
     // "IEEE Std 1003.1, 2004 Edition", "ISO/IEC 9945:2003",
     // or "Version 2 of the Single Unix Specification".
+    //
+    // It may be time to re-think the existance of this list.
+    // In the meantime, please do not add to it. The list is
+    // intended to ONLY contain flags defined by the POSIX and UNIX
+    // standards published by The Open Group, IEEE, and ISO.
     if(!strchr("aAdefgGlnoptuU", *flagptr)) not_pure_unix = 1;
 
     switch(*flagptr){
@@ -473,6 +478,15 @@ static const char *parse_sysv_option(void){
       trace("-z shows aliased MAC info\n");
       format_modifiers |= FM_M;
       break;
+    // Solaris 10 does this
+    case 'z':     /* select by zone */
+      trace("-z secects by zone\n");
+      arg=get_opt_arg();
+      if(!arg) return "List of zones (contexts, labels, whatever?) must follow -z.";
+      err=parse_list(arg, parse_zone);
+      if(err) return err;
+      selection_list->typecode = SEL_ZONE;
+      return NULL; /* can't have any more options */
 #endif
     case '-':
       return "Embedded '-' among SysV options makes no sense.";
